@@ -708,8 +708,8 @@ class Arch:
     artificial_registers_offsets = None
     artificial_registers = set()
     cpu_flag_register_offsets_and_bitmasks_map = None
-    reg_blacklist = None
-    reg_blacklist_offsets = None
+    reg_blacklist = []
+    reg_blacklist_offsets = []
     unicorn_flag_register = None
     vex_to_unicorn_map = None
     vex_cc_regs = None
